/*
 * By default this file will be overridden with every BetterTouchTool update.
 * If you make changes to this file, BTT will not update the file, however
 * features might not work correctly anymore.
 *
 * If possible only override the BTTNochBarUser.css and BTTNotchBarUser.js.
 *
 * If you absolutely need to make changes to this file, make sure to integrate the
 * changes from /Applications/BetterTouchTool.app/Contents/Resources/NotchBar/
 */

function BTTInitialize() {
  window.isInitialized = true;
  
}

const widgetTypes = {
  BTTGestureNotchBarButton: 696,
  BTTGestureNotchBarGroup: 697,
  BTTGestureNotchBarPlugin: 698,
  BTTGestureNotchBarStatusItemsWidget: 699,
  BTTGestureNotchBarCustomHTMLWidget: 700,
  BTTGestureNotchBarDateTimeWidget: 701,
  BTTGestureNotchBarWeatherWidget: 702,
  BTTGestureNotchBarRunAppleScriptWidget: 703,
  BTTGestureNotchBarNowPlayingWidget: 704


};



// this keeps the identifiers of initialized widgets
window.initializedWidgets = {};
window.widgetCache = {};

// this lists all the available configuration options
window.notchBarConfig = {
  // is this a standard or a notch bar screen?
  BTTNotchBarIsOnStandardScreen: false,

  // do we want to show the system status icon in widget mode?
  BTTNotchBarWidgetModeShowOriginalStatusIcons: false,

  // do we want to hide the left fixed status bar item in widget mode?
  BTTNotchBarWidgetModeHideLeftFixed: false,

  // do we want to hide the left scrollable status bar item in widget mode?
  BTTNotchBarWidgetModeHideLeftScrollable: false,

  // do we want to hide the right scrollable status bar item in widget mode?
  BTTNotchBarWidgetModeHideRightScrollable: false,

  // do we want to hide the right fixed status bar item in widget mode?
  BTTNotchBarWidgetModeHideRightFixed: false,

  // do we want to show the system status icon in menubar mode?
  BTTNotchBarMenubarModeShowOriginalStatusIcons: false,

  // do we want to hide the left fixed status bar item in menubar mode?
  BTTNotchBarMenubarModeHideLeftFixed: false,

  // do we want to hide the left scrollable status bar item in menubar mode?
  BTTNotchBarMenubarModeHideLeftScrollable: false,

  // do we want to hide the right scrollable status bar item in menubar mode?
  BTTNotchBarMenubarModeHideRightScrollable: false,

  // do we want to hide the right fixed status bar item in menubar mode?
  BTTNotchBarMenubarModeHideRightFixed: false,

  // show the mini menubar in widget mode?
  BTTNotchBarWidgetModeShowMiniMenubar: false,

  // show the mini statusbar area in widget mode on standard screens?
  BTTNotchBarStandardScreenWidgetModeHideMiniStatusBar: false,

  BTTNotchBarStandardMenubarAlwaysShowMenubar: false
};



function setup(config) {
  console.log("setup", config);
  if (window.repeatActionInterval) {
    clearInterval(window.repeatActionInterval);
  }
  window.notchBarConfig = config;
  console.log("did configure Notch Bar", config);

  setMenubarOnlyModeEnabled(window.menubarOnlyMode);

  document.addEventListener("long-press", function (e) {
    let triggerName = e.target.getAttribute("data-long-press-action");
    if (triggerName && window.isInitialized) {
      callBTT("trigger_named_async_without_response", {
        trigger_name: triggerName,
      });
    }
  });

  document.addEventListener("wheel", function (e) {
    if (e.deltaY < -25 && window.lastWheelDirection != "down") {
      window.lastWheelDirection = "down";
      setMenubarOnlyModeEnabled(false);
    } else if (e.deltaY > 25 && window.lastWheelDirection != "up") {
      window.lastWheelDirection = "up";
      setMenubarOnlyModeEnabled(true);
    }
  });
}

function toggleMenubarOnlyMode() {
  console.log("toggle menubar only mode");
  setMenubarOnlyModeEnabled(!window.menubarOnlyMode);
}

function setMenubarOnlyModeEnabled(mode) {
  if (!window.isInitialized) {
    setTimeout(() => {
      setMenubarOnlyModeEnabled(mode);
    }, 300);
    return;
  }
  if (mode === true) {
    activateMenubarOnlyMode();
  } else {
    activateWidgetMode();

  }
}

function activateMenubarOnlyMode() {
  if (!window.isInitialized) {
    setTimeout(() => {
      activateMenubarOnlyMode();
    }, 300);
    return;
  }
  console.log("switching to menubar only mode");
  // activate big menubar mode

  // remove the previous widget mode class
  removeClassFromElements("widget-mode", [
    "BTTNotchBar",
    "BTTNotchBarFixedLeft",
    "BTTNotchBarScrollableLeftOfNotch",
    "BTTNotchBarScrollableRightOfNotch",
    "BTTNotchBarFixedRight",
  ]);

  removeClassFromElements("widget-mode-active", ["BTTNotchBarContainer"]);

  // remove the big menubar class
  removeClassFromElements("menubar-mode", ["BTTNotchMenuBar"]);

  removeClassFromElements("mini-menubar-hidden", ["BTTNotchMenuBar"]);

  // now depending on the user settings, we either hide the complete widget bar, or
  // we leave some containers available

  // hide left fixed
  if (window.notchBarConfig.BTTNotchBarMenubarModeHideLeftFixed === true) {
    document
      .getElementById("BTTNotchBarFixedLeft")
      .classList.add("menubar-mode-notchbar");
  }
  // hide left scrollable
  if (window.notchBarConfig.BTTNotchBarMenubarModeHideLeftScrollable === true) {
    document
      .getElementById("BTTNotchBarScrollableLeftOfNotch")
      .classList.add("menubar-mode-notchbar");
  }
  // hide right scrollable
  if (
    window.notchBarConfig.BTTNotchBarMenubarModeHideRightScrollable === true
  ) {
    document
      .getElementById("BTTNotchBarScrollableRightOfNotch")
      .classList.add("menubar-mode-notchbar");
  }
  // hide right fixed
  if (window.notchBarConfig.BTTNotchBarMenubarModeHideRightFixed === true) {
    document
      .getElementById("BTTNotchBarFixedRight")
      .classList.add("menubar-mode-notchbar");
  }

  // make menubar bigger
  document.getElementById("BTTNotchMenuBar").classList.add("menubar-mode");

  // remove the BTT statusbar items if the original ones are shown
  if (
    window.notchBarConfig.BTTNotchBarMenubarModeShowOriginalStatusIcons === true
  ) {
    const statusBarItems = document.getElementsByClassName("statusbar-item");
    for (let statusBarItem of statusBarItems) {
      statusBarItem.style.display = "none";
    }
    addClassToElements("mini-statusbar-hidden", ["BTTNotchStatusBar"]);

    callBTT("set_window_level", { level: 24 });
  } else {
    callBTT("set_window_level", { level: 100 });
  }

  // TODO: option to hide the mini status bar in widget mode, even if the original status items are not shown

  window.menubarOnlyMode = true;
  callBTT('notchbar_mode_changed', {'BTTNotchBarMode': 2});

}

function activateWidgetMode() {
  if (!window.isInitialized) {
    setTimeout(() => {
      activateWidgetMode();
    }, 300);
    return;
  }
  // activate widget mode
  console.log("switching to notchbar widget mode");

  removeClassFromElements("mini-statusbar-hidden", ["BTTNotchStatusBar"]);

  if (window.notchBarConfig.BTTNotchBarWidgetModeShowMiniMenubar === true || window.notchBarConfig.BTTNotchBarStandardMenubarAlwaysShowMenubar === true) {
    removeClassFromElements("mini-menubar-hidden", ["BTTNotchMenuBar"]);
  } else {
    addClassToElements("mini-menubar-hidden", ["BTTNotchMenuBar"]);
  }

  if(window.notchBarConfig.BTTNotchBarStandardScreenWidgetModeHideMiniStatusBar === true) {
    addClassToElements("mini-statusbar-hidden", ["BTTNotchStatusBar"]);
  } else {
    removeClassFromElements("mini-statusbar-hidden", ["BTTNotchStatusBar"]);
  }

  const statusBarItems = document.getElementsByClassName("statusbar-item");
  for (let statusBarItem of statusBarItems) {
    statusBarItem.style.display = "unset";
  }

  // switch from big menubar mode to widget mode
  removeClassFromElements("menubar-mode-notchbar", [
    "BTTNotchBar",
    "BTTNotchBarFixedLeft",
    "BTTNotchBarScrollableLeftOfNotch",
    "BTTNotchBarScrollableRightOfNotch",
    "BTTNotchBarFixedRight",
  ]);
  addClassToElements("widget-mode", [
    "BTTNotchBar",
    "BTTNotchBarFixedLeft",
    "BTTNotchBarScrollableLeftOfNotch",
    "BTTNotchBarScrollableRightOfNotch",
    "BTTNotchBarFixedRight",
  ]);

  addClassToElements("widget-mode-active", ["BTTNotchBarContainer"]);

  // make menubar small again
  removeClassFromElements("menubar-mode", ["BTTNotchMenuBar"]);

  document.getElementById("BTTNotchBar").style.display = "flex";

  window.menubarOnlyMode = false;

  if (
    window.notchBarConfig.BTTNotchBarWidgetModeShowOriginalStatusIcons === true
  ) {
    callBTT("set_window_level", { level: 24 });
  } else {
    callBTT("set_window_level", { level: 100 });
  }

  callBTT('notchbar_mode_changed', {'BTTNotchBarMode': 1});

}

function addOrUpdateStatusBarItem(containerID, base64, x, y, elementID) {

  const containerElement = document.getElementById(containerID);
  if (!containerElement) {
    return;
  }

  let div = document.getElementById(elementID);
  if (!div) {
    div = document.createElement("div");
    div.alt = "status icon";
    div.id = elementID;
    div.setAttribute("data-x", x);
    div.setAttribute("data-use-for-highlight", "true");
    div.classList.add("statusbar-item");
    containerElement.appendChild(div);
    div.onmouseup = function () {
      callBTT("click_status_item", { id: elementID });
    };
  }

  let img = document.getElementById(elementID + "img");

  // create html image tag using js
  if (!img) {
    img = document.createElement("img");
    img.alt = "status icon";
    img.setAttribute("data-x", x);
    img.id = elementID + "img";
    img.setAttribute("x-apple-data-detectors", "false");
    // set class for img tag
    img.classList.add("statusbar-item-image");
   

    div.appendChild(img);
  }

  let children = Array.prototype.slice.call(containerElement.children, 0);

  children.sort(function (a, b) {
    var aord = +a.getAttribute("data-x");
    var bord = +b.getAttribute("data-x");
    return aord - bord;
  });

  containerElement.innerHTML = "";

  for (var i = 0, l = children.length; i < l; i++) {
    containerElement.appendChild(children[i]);
  }

  if (base64 && base64.length > 0) {
  
    img.src = `data:image/png;base64,${base64}`;
  }
}

function updateDisplayValue(elementID, widgetType, value, imageBase64, json, invert) {
  json = JSON.parse(decodeBase64(json));
  value = decodeBase64(value);
  const element = document.getElementById(elementID);
  if (element) {
    switch (widgetType) {
      case widgetTypes.BTTGestureNotchBarNowPlayingWidget:
      case widgetTypes.BTTGestureNotchBarRunAppleScriptWidget:
      case widgetTypes.BTTGestureNotchBarWeatherWidget:
      case widgetTypes.BTTGestureNotchBarDateTimeWidget:
      case widgetTypes.BTTGestureNotchBarButton:
        updateDisplayValueForButtonElement(element, value, imageBase64, json, invert);
        break;

      default:
        break;
    }
  }
}

/*
This function is called by BTT to add or update the standard Notch Bar buttons
*/
function addOrUpdateButton(elementID, jsonDescription) {
  jsonDescription = JSON.parse(decodeBase64(jsonDescription));

  if (window.repeatActionInterval) {
    clearInterval(window.repeatActionInterval);
    window.repeatActionInterval = null;
  }

  const containerID = getPlacementContainer(jsonDescription);

  let btn = document.getElementById(elementID);
  let img;
  const containerElement = document.getElementById(containerID);
  if (!containerElement) {
    console.log("container not found", containerID);
    return;
  }
  if (!btn || +btn.getAttribute("data-order") != jsonDescription.order) {
    if (btn) {
      removeElementWithID(elementID);
    }

    btn = document.createElement("div");
    btn.id = elementID;
    btn.alt = jsonDescription.BTTTouchBarButtonName;
    btn.classList.add("button");
    btn.classList.add("needs-reset-on-update");

    btn.setAttribute("data-order", jsonDescription.order);
    btn.setAttribute("x-apple-data-detectors", "false");
    btn.setAttribute("data-use-for-highlight", "true");

    if (
      jsonDescription.BTTNotchBarActionRepeatRate &&
      jsonDescription.BTTNotchBarActionRepeatRate > 0.01
    ) {
      btn.onmousedown = () => {
        callBTT("widget_clicked", { uuid: elementID });

        const repeatDelay =
          jsonDescription.BTTNotchBarActionRepeatDelay > 0.01
            ? jsonDescription.BTTNotchBarActionRepeatDelay
            : 0;

        setTimeout(() => {
          window.repeatActionInterval = setInterval(() => {
            callBTT("widget_clicked", {
              uuid: elementID,
            });
          }, jsonDescription.BTTNotchBarActionRepeatRate * 1000);
        }, repeatDelay * 1000);
      };

      btn.onmouseup = () => {
        if (window.repeatActionInterval) {
          clearInterval(window.repeatActionInterval);
          window.repeatActionInterval = null;
        }
      };
    } else {
      btn.onclick = function () {
        if (window.repeatActionInterval) {
          clearInterval(window.repeatActionInterval);
          window.repeatActionInterval = null;
        }

        if (
          !jsonDescription.BTTNotchBarActionRepeatDelay ||
          jsonDescription.BTTNotchBarActionRepeatDelay < 0.01 ||
          jsonDescription.BTTTouchBarLongPressActionName ||
          jsonDescription.BTTTouchBarLongPressActionName.length > 0
        ) {
        
          if(jsonDescription.BTTTriggerType == 630) {
            callBTT("open_notchbar_group", { uuid: elementID });

          } else {
            callBTT("widget_clicked", { uuid: elementID });
          }
        }
      };
    }

    // custom styling

    containerElement.appendChild(btn);

    let children = Array.prototype.slice.call(containerElement.children, 0);

    children.sort(function (a, b) {
      var aord = +a.getAttribute("data-order");
      var bord = +b.getAttribute("data-order");
      return aord - bord;
    });

    containerElement.innerHTML = "";

    for (var i = 0, l = children.length; i < l; i++) {
      containerElement.appendChild(children[i]);
    }
  }

  addDefaultVariablesToElement(btn, jsonDescription);

  const userStyles = jsonDescription.BTTNotchBarCSSStyles;
  const userClass = jsonDescription.BTTNotchBarCSSClass;
  if (userStyles && userStyles.length > 0) {
    btn.style.cssText += ";" + userStyles;
  }

  if (userClass && userClass.length > 0) {
    btn.classList.add(userClass);
  }

  //longpress
  if (
    jsonDescription.BTTTouchBarLongPressActionName &&
    jsonDescription.BTTTouchBarLongPressActionName.length > 0
  ) {
    const repeatDelay =
      jsonDescription.BTTNotchBarActionRepeatDelay > 0.01
        ? jsonDescription.BTTNotchBarActionRepeatDelay
        : 0;
    btn.setAttribute("data-long-press-delay", repeatDelay * 1000);
    btn.setAttribute(
      "data-long-press-action",
      jsonDescription.BTTTouchBarLongPressActionName
    );
  }

  if (jsonDescription.BTTIconData) {
    img = btn.querySelector(".btn-img");
    if (!img) {
      img = document.createElement("img");
      btn.appendChild(img);
    }
    img.src = `data:image/png;base64,${jsonDescription.BTTIconData}`;
    img.className = "btn-img";
    img.style.width = `${jsonDescription.BTTTouchBarItemIconWidth}px`;
    img.style.height = `${jsonDescription.BTTTouchBarItemIconHeight}px`;

    if (jsonDescription.BTTTouchBarIconInvert === 1 || jsonDescription.BTTTouchBarIconInvert === true) {
      img.style.filter = "invert(100%)";
    }
  }

  if (jsonDescription.BTTTouchBarButtonName || jsonDescription.BTTGroupName) {
    updateDisplayValueForButtonElement(
      btn,
      jsonDescription.BTTTouchBarButtonName ? jsonDescription.BTTTouchBarButtonName : jsonDescription.BTTGroupName,
      jsonDescription.BTTIconData,
      jsonDescription
    );
  }
}

/*
 * This function is called by BTT to add HTML/JS widgets.
 */
function addHTMLWidget(jsonDescription, elementID, order) {
  console.log("addHTMLWidget");
  let didInitialize = false;

  jsonDescription = JSON.parse(decodeBase64(jsonDescription));
  const containerID = getPlacementContainer(jsonDescription);

  let btn = document.getElementById(elementID);
  if (!btn || +btn.getAttribute("data-order") != order) {
    if (btn) {
      removeElementWithID(elementID);
    }

    btn = document.createElement("div");
    btn.id = elementID;
    btn.className = "notchbar-html-widget";
    btn.setAttribute("data-use-for-highlight", "true");
    btn.classList.add("needs-reset-on-update");

    /*
   BTTNotchBarJSFunctionOnHighlight
     BTTNotchBarJSFunctionOnRefresh
     BTTNotchBarJSFunctionOnLoad
     BTTNotchBarCSS
     BTTNotchBarJS
     BTTNotchBarHTML*/

    // add html
    btn.innerHTML = jsonDescription.BTTNotchBarHTML;
    if (window.initializedWidgets[elementID] === undefined) {
      // add css
      if (
        jsonDescription.BTTNotchBarCSS &&
        jsonDescription.BTTNotchBarCSS.length > 0
      ) {
        const style = document.createElement("style");
        style.textContent = jsonDescription.BTTNotchBarCSS;
        document.head.append(style);
      }

      // add java script
      if (
        jsonDescription.BTTNotchBarJS &&
        jsonDescription.BTTNotchBarJS.length > 0
      ) {
        var head = document.getElementsByTagName("head")[0];
        var script = document.createElement("script");
        script.type = "text/javascript";
        script.id = elementID+'script';

        if (
          jsonDescription.BTTNotchBarJSFunctionOnLoad &&
          jsonDescription.BTTNotchBarJSFunctionOnLoad.length > 0
        ) {
          setTimeout(() => {
            eval(jsonDescription.BTTNotchBarJSFunctionOnLoad);
          }, 1);
        }

        script.text = jsonDescription.BTTNotchBarJS;
        head.appendChild(script);
        didInitialize = true;
      } else {
        const script = document.getElementById(elementID+'script');
        if(script.text != jsonDescription.BTTNotchBarJS) {
          script.text = jsonDescription.BTTNotchBarJS;
        }
      }

      if (
        jsonDescription.BTTNotchBarJSFunctionOnHighlight &&
        jsonDescription.BTTNotchBarJSFunctionOnHighlight.length > 0
      ) {
        btn.setAttribute(
          "data-on-highlight",
          jsonDescription.BTTNotchBarJSFunctionOnHighlight
        );
      }
    }
    addDefaultVariablesToElement(btn, jsonDescription);
    btn.setAttribute("data-order", order);

    window.initializedWidgets[elementID] = true;
  }

  // refresh js

  if (didInitialize === false) {
    if (
      jsonDescription.BTTNotchBarJSFunctionOnRefresh &&
      jsonDescription.BTTNotchBarJSFunctionOnRefresh.length > 0
    ) {
      setTimeout(() => {
        eval(jsonDescription.BTTNotchBarJSFunctionOnRefresh);
      }, 1);
    }
  }

  const containerElement = document.getElementById(containerID);
  if (containerElement) {
    containerElement.appendChild(btn);

    let children = Array.prototype.slice.call(containerElement.children, 0);

    children.sort(function (a, b) {
      var aord = +a.getAttribute("data-order");
      var bord = +b.getAttribute("data-order");
      return aord - bord;
    });

    containerElement.innerHTML = "";

    for (var i = 0, l = children.length; i < l; i++) {
      containerElement.appendChild(children[i]);
    }
  } else {
    console.error("container element does not exist", containerID);
  }
}

function exitFullScreen() {
  console.log("exit fullscreen");
  const exitFullScreen = { BTTPredefinedActionType: 112 };
  callBTT("trigger_action", { json: JSON.stringify(exitFullScreen) });
}

function didEnterFullscreenMode() {
  console.log("did enter fullscreen mode");
  removeClassFromElements("fullscreen-toggle-hidden", ["BTTFullscreenToggle"]);
  addClassToElements("fullscreen-toggle-visible", ["BTTFullscreenToggle"]);
}

function didLeaveFullscreenMode() {
  console.log("did leave fullscreen mode");
  removeClassFromElements("fullscreen-toggle-visible", ["BTTFullscreenToggle"]);
  addClassToElements("fullscreen-toggle-hidden", ["BTTFullscreenToggle"]);
}
